@@ -23,7 +23,7 @@
 NULL
 
 #' @export
-#' @rdname simulate_dbi
+#' @rdname backend-hive
 simulate_hive <- function() simulate_dbi("Hive")
 
 #' @export
@@ -54,7 +54,34 @@ sql_translation.Hive <- function(con) {
     sql_translator(.parent = base_odbc_win,
       var = win_aggregate("VARIANCE"),
       quantile = sql_quantile("PERCENTILE", window = TRUE),
-      median = sql_median("PERCENTILE", window = TRUE)
+      median = sql_median("PERCENTILE", window = TRUE),
+      first = function(x, order_by = NULL, na_rm = FALSE) {
+        sql_nth(
+          x = x,
+          n = 1L,
+          order_by = order_by,
+          na_rm = na_rm,
+          ignore_nulls = "bool"
+        )
+      },
+      last = function(x, order_by = NULL, na_rm = FALSE) {
+        sql_nth(
+          x = x,
+          n = Inf,
+          order_by = order_by,
+          na_rm = na_rm,
+          ignore_nulls = "bool"
+        )
+      },
+      nth = function(x, n, order_by = NULL, na_rm = FALSE) {
+        sql_nth(
+          x = x,
+          n = n,
+          order_by = order_by,
+          na_rm = na_rm,
+          ignore_nulls = "bool"
+        )
+      },
     )
   )
 }
@@ -62,28 +89,20 @@ sql_translation.Hive <- function(con) {
 #' @export
 sql_table_analyze.Hive <- function(con, table, ...) {
   # https://cwiki.apache.org/confluence/display/Hive/StatsDev
-  build_sql(
-    "ANALYZE TABLE ", as.sql(table, con = con), " COMPUTE STATISTICS",
-    con = con
-  )
-}
-
-#' @export
-last_value_sql.Hive <- function(con, x) {
-  translate_sql(last_value(!!x, TRUE), con = con)
+  glue_sql2(con, "ANALYZE TABLE {.tbl table} COMPUTE STATISTICS")
 }
 
 #' @export
 sql_query_set_op.Hive <- function(con, x, y, method, ..., all = FALSE, lvl = 0) {
-  # SQLite does not allow parentheses
+  check_bool(all)
+  # parentheses are not allowed
   method <- paste0(method, if (all) " ALL")
-  # `x` and `y` already have the correct indent, so use `build_sql()` instead
-  # of `sql_format_clauses()`
-  build_sql(
-    x, "\n",
-    indent_lvl(style_kw(method), lvl = lvl), "\n",
-    y,
-    con = con
+  glue_sql2(
+    con,
+    "\n", # dummy line to protect indent
+    "{x}\n",
+    lvl_indent(lvl), "{.kw method}\n",
+    y
   )
 }
 
@@ -92,4 +111,4 @@ supports_window_clause.Hive <- function(con) {
   TRUE
 }
 
-globalVariables("regexp_replace")
+utils::globalVariables("regexp_replace")
