@@ -34,6 +34,9 @@
 #'   The default, "never", is how databases usually work. `"na"` makes
 #'   the joins behave like the dplyr join functions, [merge()], [match()],
 #'   and `%in%`.
+#' @param multiple,unmatched Unsupported in database backends. As a workaround
+#'   for multiple use a unique key and for unmatched a foreign key constraint.
+#' @param relationship Unsupported in database backends.
 #' @param x_as,y_as Alias to use for `x` resp. `y`. Defaults to `"LHS"` resp.
 #'   `"RHS"`
 #' @inherit arrange.tbl_lazy return
@@ -67,23 +70,38 @@ NULL
 #' @rdname join.tbl_sql
 #' @export
 #' @importFrom dplyr inner_join
-inner_join.tbl_lazy <- function(x, y, by = NULL, copy = FALSE,
+inner_join.tbl_lazy <- function(x,
+                                y,
+                                by = NULL,
+                                copy = FALSE,
                                 suffix = NULL,
-                                auto_index = FALSE, ...,
-                                sql_on = NULL, na_matches = c("never", "na"),
-                                x_as = NULL, y_as = NULL) {
+                                ...,
+                                keep = NULL,
+                                na_matches = c("never", "na"),
+                                multiple = NULL,
+                                unmatched = "drop",
+                                relationship = NULL,
+                                sql_on = NULL,
+                                auto_index = FALSE,
+                                x_as = NULL,
+                                y_as = NULL) {
+  check_dots_empty()
   x$lazy_query <- add_join(
-    x, y,
-    "inner",
+    x,
+    y,
+    type = "inner",
     by = by,
-    sql_on = sql_on,
     copy = copy,
     suffix = suffix,
-    auto_index = auto_index,
+    keep = keep,
     na_matches = na_matches,
+    multiple = multiple,
+    unmatched = unmatched,
+    relationship = relationship,
+    sql_on = sql_on,
+    auto_index = auto_index,
     x_as = x_as,
-    y_as = y_as,
-    ...
+    y_as = y_as
   )
 
   x
@@ -92,23 +110,38 @@ inner_join.tbl_lazy <- function(x, y, by = NULL, copy = FALSE,
 #' @rdname join.tbl_sql
 #' @export
 #' @importFrom dplyr left_join
-left_join.tbl_lazy <- function(x, y, by = NULL, copy = FALSE,
+left_join.tbl_lazy <- function(x,
+                               y,
+                               by = NULL,
+                               copy = FALSE,
                                suffix = NULL,
-                               auto_index = FALSE, ...,
-                               sql_on = NULL, na_matches = c("never", "na"),
-                               x_as = NULL, y_as = NULL) {
+                               ...,
+                               keep = NULL,
+                               na_matches = c("never", "na"),
+                               multiple = NULL,
+                               unmatched = "drop",
+                               relationship = NULL,
+                               sql_on = NULL,
+                               auto_index = FALSE,
+                               x_as = NULL,
+                               y_as = NULL) {
+  check_dots_empty()
   x$lazy_query <- add_join(
-    x, y,
-    "left",
+    x,
+    y,
+    type = "left",
     by = by,
-    sql_on = sql_on,
     copy = copy,
     suffix = suffix,
-    auto_index = auto_index,
+    keep = keep,
     na_matches = na_matches,
+    multiple = multiple,
+    unmatched = unmatched,
+    relationship = relationship,
+    sql_on = sql_on,
+    auto_index = auto_index,
     x_as = x_as,
-    y_as = y_as,
-    ...
+    y_as = y_as
   )
 
   x
@@ -117,23 +150,38 @@ left_join.tbl_lazy <- function(x, y, by = NULL, copy = FALSE,
 #' @rdname join.tbl_sql
 #' @export
 #' @importFrom dplyr right_join
-right_join.tbl_lazy <- function(x, y, by = NULL, copy = FALSE,
+right_join.tbl_lazy <- function(x,
+                                y,
+                                by = NULL,
+                                copy = FALSE,
                                 suffix = NULL,
-                                auto_index = FALSE, ...,
-                                sql_on = NULL, na_matches = c("never", "na"),
-                               x_as = NULL, y_as = NULL) {
+                                ...,
+                                keep = NULL,
+                                na_matches = c("never", "na"),
+                                multiple = NULL,
+                                unmatched = "drop",
+                                relationship = NULL,
+                                sql_on = NULL,
+                                auto_index = FALSE,
+                                x_as = NULL,
+                                y_as = NULL) {
+  check_dots_empty()
   x$lazy_query <- add_join(
-    x, y,
-    "right",
+    x,
+    y,
+    type = "right",
     by = by,
-    sql_on = sql_on,
     copy = copy,
     suffix = suffix,
-    auto_index = auto_index,
+    keep = keep,
     na_matches = na_matches,
+    multiple = multiple,
+    unmatched = unmatched,
+    relationship = relationship,
+    sql_on = sql_on,
+    auto_index = auto_index,
     x_as = x_as,
-    y_as = y_as,
-    ...
+    y_as = y_as
   )
 
   x
@@ -142,20 +190,61 @@ right_join.tbl_lazy <- function(x, y, by = NULL, copy = FALSE,
 #' @rdname join.tbl_sql
 #' @export
 #' @importFrom dplyr full_join
-full_join.tbl_lazy <- function(x, y, by = NULL, copy = FALSE,
+full_join.tbl_lazy <- function(x,
+                               y,
+                               by = NULL,
+                               copy = FALSE,
                                suffix = NULL,
-                               auto_index = FALSE, ...,
-                               sql_on = NULL, na_matches = c("never", "na"),
-                               x_as = NULL, y_as = NULL) {
+                               ...,
+                               keep = NULL,
+                               na_matches = c("never", "na"),
+                               multiple = NULL,
+                               relationship = NULL,
+                               sql_on = NULL,
+                               auto_index = FALSE,
+                               x_as = NULL,
+                               y_as = NULL) {
+  check_dots_empty()
   x$lazy_query <- add_join(
-    x, y,
-    "full",
+    x,
+    y,
+    type = "full",
     by = by,
-    sql_on = sql_on,
     copy = copy,
     suffix = suffix,
-    auto_index = auto_index,
+    keep = keep,
     na_matches = na_matches,
+    multiple = multiple,
+    relationship = relationship,
+    sql_on = sql_on,
+    auto_index = auto_index,
+    x_as = x_as,
+    y_as = y_as
+  )
+
+  x
+}
+
+#' @rdname join.tbl_sql
+#' @export
+#' @importFrom dplyr cross_join
+cross_join.tbl_lazy <- function(x,
+                                y,
+                                ...,
+                                copy = FALSE,
+                                suffix = c(".x", ".y"),
+                                x_as = NULL,
+                                y_as = NULL) {
+  x$lazy_query <- add_join(
+    x, y,
+    "cross",
+    by = character(),
+    sql_on = NULL,
+    copy = copy,
+    suffix = suffix,
+    auto_index = FALSE,
+    keep = NULL,
+    na_matches = "never",
     x_as = x_as,
     y_as = y_as,
     ...
@@ -167,21 +256,28 @@ full_join.tbl_lazy <- function(x, y, by = NULL, copy = FALSE,
 #' @rdname join.tbl_sql
 #' @export
 #' @importFrom dplyr semi_join
-semi_join.tbl_lazy <- function(x, y, by = NULL, copy = FALSE,
-                               auto_index = FALSE, ...,
-                               sql_on = NULL, na_matches = c("never", "na"),
-                               x_as = NULL, y_as = NULL) {
+semi_join.tbl_lazy <- function(x,
+                               y,
+                               by = NULL,
+                               copy = FALSE,
+                               ...,
+                               na_matches = c("never", "na"),
+                               sql_on = NULL,
+                               auto_index = FALSE,
+                               x_as = NULL,
+                               y_as = NULL) {
+  check_dots_empty()
   x$lazy_query <- add_semi_join(
-    x, y,
+    x,
+    y,
     anti = FALSE,
     by = by,
-    sql_on = sql_on,
     copy = copy,
-    auto_index = auto_index,
     na_matches = na_matches,
+    sql_on = sql_on,
+    auto_index = auto_index,
     x_as = x_as,
-    y_as = y_as,
-    ...
+    y_as = y_as
   )
 
   x
@@ -190,42 +286,72 @@ semi_join.tbl_lazy <- function(x, y, by = NULL, copy = FALSE,
 #' @rdname join.tbl_sql
 #' @export
 #' @importFrom dplyr anti_join
-anti_join.tbl_lazy <- function(x, y, by = NULL, copy = FALSE,
-                               auto_index = FALSE, ...,
-                               sql_on = NULL, na_matches = c("never", "na"),
-                               x_as = NULL, y_as = NULL) {
+anti_join.tbl_lazy <- function(x,
+                               y,
+                               by = NULL,
+                               copy = FALSE,
+                               ...,
+                               na_matches = c("never", "na"),
+                               sql_on = NULL,
+                               auto_index = FALSE,
+                               x_as = NULL,
+                               y_as = NULL) {
+  check_dots_empty()
   x$lazy_query <- add_semi_join(
-    x, y,
+    x,
+    y,
     anti = TRUE,
     by = by,
-    sql_on = sql_on,
     copy = copy,
-    auto_index = auto_index,
     na_matches = na_matches,
+    sql_on = sql_on,
+    auto_index = auto_index,
     x_as = x_as,
-    y_as = y_as,
-    ...
+    y_as = y_as
   )
 
   x
 }
 
-
-add_join <- function(x, y, type, by = NULL, sql_on = NULL, copy = FALSE,
+add_join <- function(x,
+                     y,
+                     type,
+                     by = NULL,
+                     copy = FALSE,
                      suffix = NULL,
-                     auto_index = FALSE,
+                     keep = NULL,
                      na_matches = "never",
+                     multiple = NULL,
+                     unmatched = "drop",
+                     relationship = NULL,
+                     sql_on = NULL,
+                     auto_index = FALSE,
                      x_as = NULL,
                      y_as = NULL,
                      call = caller_env()) {
-  if (!is.null(sql_on)) {
-    by <- list(x = character(0), y = character(0), on = unclass(sql_on))
-  } else if (identical(type, "full") && identical(by, character())) {
-    type <- "cross"
-    by <- list(x = character(0), y = character(0))
-  } else {
-    by <- dplyr::common_by(by, x, y)
-  }
+  x_names <- tbl_vars(x)
+  y_names <- tbl_vars(y)
+
+  by <- join_prepare_by(
+    by = by,
+    sql_on = sql_on,
+    type = type,
+    x_names = x_names,
+    y_names = y_names,
+    error_call = call
+  )
+  type <- by$type
+
+  check_join_by_supported(by, call = call)
+
+  check_join_multiple(multiple, by, call = call)
+  check_join_unmatched(unmatched, by, call = call)
+  check_unsupported_arg(
+    relationship,
+    allowed = "many-to-many",
+    allow_null = TRUE,
+    call = call
+  )
 
   y <- auto_copy(
     x, y,
@@ -234,53 +360,71 @@ add_join <- function(x, y, type, by = NULL, sql_on = NULL, copy = FALSE,
   )
 
   suffix <- suffix %||% sql_join_suffix(x$src$con, suffix)
-  suffix <- check_suffix(suffix, call)
+  vars <- join_cols(
+    x_names = x_names,
+    y_names = y_names,
+    by = by,
+    suffix = suffix,
+    keep = keep,
+    error_call = call
+  )
+
   na_matches <- arg_match(na_matches, c("na", "never"), error_call = call)
 
   # the table alias can only be determined after `select()` was inlined.
   # This works even though `by` is used in `join_inline_select()` and updated
   # because this does not touch `by$x_as` and `by$y_as`.
-  # TODO can this be really be done before inlining?
-  join_alias <- make_join_aliases(x_as, y_as, sql_on, call)
+  join_alias <- make_join_aliases(x$src$con, x_as, y_as, sql_on, call)
 
   inline_result <- join_inline_select(x$lazy_query, by$x, by$on)
   x_lq <- inline_result$lq
   x_vars <- inline_result$vars
-  by_x_org <- by$x
   by$x <- inline_result$by
 
-  new_query <- join_needs_new_query(x$lazy_query, join_alias, type)
-  if (new_query) {
-    x_join_vars <- tibble(
-      name = op_vars(x),
-      table = list(1L),
-      var = as.list(x_vars)
-    )
-    table_id <- 2L
-  } else {
-    x_join_vars <- x_lq$vars
-    table_id <- length(x_lq$table_names) + 1L
-  }
-
   inline_result <- join_inline_select(y$lazy_query, by$y, by$on)
   y_lq <- inline_result$lq
   y_vars <- inline_result$vars
   by$y <- inline_result$by
 
-  y_join_vars <- tibble(
-    name = op_vars(y),
-    table = list(table_id),
-    var = as.list(y_vars)
-  )
+  if (type %in% c("full", "right")) {
+    vars <- rf_join_vars(
+      x_vars = x_vars,
+      y_vars = y_vars,
+      vars_info = vars,
+      keep = keep,
+      condition = by$condition,
+      by_y = by$y,
+      type = type
+    )
 
+    table_names_x <- make_table_names(join_alias$x, x_lq)
+    table_names_y <- make_table_names(join_alias$y, y_lq)
+    out <- lazy_rf_join_query(
+      x = x_lq,
+      y = y_lq,
+      type = type,
+      by = list(
+        x = by$x,
+        y = by$y,
+        condition = by$condition,
+        on = sql(by$on),
+        na_matches = na_matches
+      ),
+      table_names = vctrs::vec_rbind(table_names_x, table_names_y),
+      vars = vars
+    )
+
+    return(out)
+  }
+
+  new_query <- join_needs_new_query(x$lazy_query, join_alias, type)
   vars <- multi_join_vars(
-    x_join_vars = x_join_vars,
-    y_join_vars = y_join_vars,
-    by_x_org = by_x_org,
-    by_y = by$y,
-    type = type,
-    table_id = table_id,
-    suffix = suffix
+    x_lq = x_lq,
+    x_vars = x_vars,
+    y_vars = y_vars,
+    new_query = new_query,
+    vars_info = vars,
+    error_call = call
   )
 
   joins_data <- new_joins_data(
@@ -346,10 +490,6 @@ join_needs_new_query <- function(x_lq, join_alias, type) {
     return(TRUE)
   }
 
-  if (type %in% c("full", "right")) {
-    return(TRUE)
-  }
-
   x_as <- join_alias$x
   y_as <- join_alias$y
 
@@ -374,64 +514,110 @@ join_needs_new_query <- function(x_lq, join_alias, type) {
   FALSE
 }
 
-multi_join_vars <- function(x_join_vars,
-                            y_join_vars,
-                            by_x_org,
-                            by_y,
-                            type,
-                            table_id,
-                            suffix,
-                            call) {
-  # Remove join keys from y
-  y_join_idx <- vctrs::vec_match(by_y, unlist(y_join_vars$var))
-  if (!is_empty(y_join_idx)) {
-    y_join_vars <- vctrs::vec_slice(y_join_vars, -y_join_idx)
+multi_join_vars <- function(x_lq,
+                            x_vars,
+                            y_vars,
+                            new_query,
+                            vars_info,
+                            error_call) {
+  if (new_query) {
+    x_join_vars <- tibble(
+      name = names(vars_info$x$out),
+      table = 1L,
+      var = vctrs::vec_slice(x_vars, vars_info$x$out)
+    )
+    table_id <- 2L
+  } else {
+    x_join_vars <- vctrs::vec_slice(x_lq$vars, vars_info$x$out)
+    x_join_vars$name <- names(vars_info$x$out)
+    table_id <- vctrs::vec_size(x_lq$table_names) + 1L
   }
-  x_names <- x_join_vars$name
-  y_names <- y_join_vars$name
 
-  # Add suffix where needed
-  x_join_vars$name <- add_suffixes(x_names, y_names, suffix$x)
-  y_join_vars$name <- add_suffixes(y_names, x_names, suffix$y)
+  y_join_vars <- tibble(
+    name = names(vars_info$y$out),
+    table = table_id,
+    var = vctrs::vec_slice(y_vars, vars_info$y$out)
+  )
 
-  if (type %in% c("left", "inner")) {
-    # use all variables from `x` as is
-    # use non-join variables from `y`
-  } else if (type == "right") {
-    # Careful: this relies on the assumption that right_join()` starts a new query
+  vctrs::vec_rbind(x_join_vars, y_join_vars, .error_call = error_call)
+}
+
+rf_join_vars <- function(x_vars,
+                         y_vars,
+                         vars_info,
+                         keep,
+                         condition,
+                         by_y,
+                         type,
+                         call) {
+  x_join_vars <- tibble(
+    name = names(vars_info$x$out),
+    x = vctrs::vec_slice(x_vars, vars_info$x$out),
+    y = NA_character_
+  )
+
+  y_join_vars <- tibble(
+    name = names(vars_info$y$out),
+    x = NA_character_,
+    y = vctrs::vec_slice(y_vars, vars_info$y$out)
+  )
+
+  keep <- keep %||% (condition != "==")
+  idx <- vars_info$x$key[!keep]
+  if (type == "right") {
     # `x`: non-join variables; `y`: all variables
-    # -> must update table id of `x` join vars
-    x_join_vars$table[x_join_vars$name %in% by_x_org] <- list(table_id)
-    idx <- vctrs::vec_match(by_x_org, x_join_vars$name)
-    x_join_vars$var[idx] <- as.list(by_y)
-  } else if (type == "full") {
-    # Careful: this relies on the assumption that `full_join()` starts a new query
-    idx <- vctrs::vec_match(by_x_org, x_join_vars$name)
-    x_join_vars$table[idx] <- purrr::map(
-      x_join_vars$table[idx],
-      ~ c(.x, table_id)
-    )
-    x_join_vars$var[idx] <- purrr::map2(
-      x_join_vars$var[idx], by_y,
-      ~ c(.x, .y)
-    )
-  } else if (type == "cross") {
-    # -> simply append `y_rows`
+    x_join_vars$x[idx] <- NA
   }
+  x_join_vars$y[idx] <- by_y[!keep]
 
   vctrs::vec_rbind(x_join_vars, y_join_vars)
 }
 
+
+join_prepare_by <- function(by,
+                            sql_on,
+                            type,
+                            x_names,
+                            y_names,
+                            error_call,
+                            env = caller_env(2),
+                            user_env = caller_env(3)) {
+  if (identical(by, character()) && is.null(sql_on)) {
+    if (type != "cross") {
+      lifecycle::deprecate_warn(
+        when = "1.1.0",
+        what = I("Using `by = character()` to perform a cross join"),
+        with = "cross_join()",
+        env = env,
+        user_env = user_env
+      )
+    }
+    type <- "cross"
+  }
+
+  if (!is.null(sql_on)) {
+    by <- list(x = character(0), y = character(0), on = unclass(sql_on))
+  } else if (type == "cross") {
+    by <- list(x = character(0), y = character(0))
+  } else if (is_null(by)) {
+    by <- join_by_common(x_names, y_names, error_call = error_call)
+  } else {
+    by <- dbplyr_as_join_by(by, error_call = error_call)
+  }
+
+  by$type <- type
+  by
+}
+
 new_joins_data <- function(x_lq, y_lq, new_query, type, by, na_matches) {
   if (new_query) {
     by_x_table_id <- rep_along(by$x, 1L)
   } else {
     idx <- vctrs::vec_match(by$x, x_lq$vars$name)
-    stopifnot(all(vctrs::list_sizes(x_lq$vars$var[idx]) == 1))
 
     # need to fix `by$x` in case it was renamed in an inlined select
-    by$x <- unlist(x_lq$vars$var)[idx]
-    by_x_table_id <- unlist(x_lq$vars$table)[idx]
+    by$x <- x_lq$vars$var[idx]
+    by_x_table_id <- x_lq$vars$table[idx]
   }
 
   tibble(
@@ -439,24 +625,40 @@ new_joins_data <- function(x_lq, y_lq, new_query, type, by, na_matches) {
     type = type,
     by_x_table_id = list(by_x_table_id),
     by = list(list(
-      x = ident(by$x),
-      y = ident(by$y),
+      x = by$x,
+      y = by$y,
+      condition = by$condition,
       on = sql(by$on),
       na_matches = na_matches
     ))
   )
 }
 
-add_semi_join <- function(x, y, anti = FALSE, by = NULL, sql_on = NULL, copy = FALSE,
-                          auto_index = FALSE, na_matches = "never",
-                          x_as = NULL, y_as = NULL,
+add_semi_join <- function(x,
+                          y,
+                          anti = FALSE,
+                          by = NULL,
+                          sql_on = NULL,
+                          copy = FALSE,
+                          auto_index = FALSE,
+                          na_matches = "never",
+                          x_as = NULL,
+                          y_as = NULL,
                           call = caller_env()) {
+  x_names <- tbl_vars(x)
+  y_names <- tbl_vars(y)
+
   if (!is.null(sql_on)) {
     by <- list(x = character(0), y = character(0), on = sql(sql_on))
+  } else if (is_null(by)) {
+    by <- join_by_common(x_names, y_names, error_call = call)
   } else {
-    by <- dplyr::common_by(by, x, y)
+    by <- dbplyr_as_join_by(by, error_call = call)
   }
 
+  check_join_by_supported(by, call = call)
+  na_matches <- arg_match(na_matches, c("na", "never"), error_call = call)
+
   y <- auto_copy(
     x, y, copy,
     indexes = if (auto_index) list(by$y)
@@ -464,36 +666,79 @@ add_semi_join <- function(x, y, anti = FALSE, by = NULL, sql_on = NULL, copy = F
 
   inline_result <- join_inline_select(x$lazy_query, by$x, sql_on)
   x_lq <- inline_result$lq
+  x_vars <- inline_result$vars
   by$x <- inline_result$by
+
+  inline_result <- semi_join_inline_select(y$lazy_query, by$y, sql_on)
+  y_lq <- inline_result$lq
+  by$y <- inline_result$by
+  where <- inline_result$where
+
   vars <- tibble(
     name = op_vars(x),
-    var = inline_result$vars
+    var = x_vars
   )
+  by$na_matches <- na_matches
 
   # the table alias can only be determined after `select()` was inlined
-  join_alias <- make_join_aliases(x_as, y_as, sql_on, call)
+  join_alias <- make_join_aliases(x$src$con, x_as, y_as, sql_on, call)
 
-  x_alias <- make_table_names(join_alias$x, x_lq)
-  y_alias <- make_table_names(join_alias$y, y)
-  by[c("x_as", "y_as")] <- join_two_table_alias(
-    c(x_alias$name, y_alias$name),
-    c(x_alias$from, y_alias$from)
+  aliases <- vctrs::vec_rbind(
+    make_table_names(join_alias$x, x_lq),
+    make_table_names(join_alias$y, y_lq)
   )
-  by$x_as <- ident(by$x_as)
-  by$y_as <- ident(by$y_as)
+  names <- generate_join_table_names(aliases, remote_con(x))
+  by$x_as <- names[1]
+  by$y_as <- names[2]
 
   lazy_semi_join_query(
     x_lq,
-    y$lazy_query,
+    y_lq,
     vars = vars,
     anti = anti,
     by = by,
-    na_matches = na_matches,
+    where = where,
     call = call
   )
 }
 
-make_join_aliases <- function(x_as, y_as, sql_on, call) {
+can_inline_semi_join <- function(x) {
+  is_lazy_select_query_simple(
+    x,
+    ignore_where = TRUE,
+    ignore_group_by = TRUE,
+    select = "projection"
+  )
+}
+
+semi_join_inline_select <- function(lq, by, on) {
+  if (is_empty(on) && can_inline_semi_join(lq)) {
+    vars <- purrr::map_chr(lq$select$expr, as_string)
+
+    idx <- vctrs::vec_match(lq$select$name, by)
+    by <- vctrs::vec_assign(by, idx, vars)
+
+    lq_org <- lq
+    lq <- lq$x
+    lq$group_vars <- op_grps(lq_org)
+    lq$order_vars <- op_sort(lq_org)
+    lq$frame <- op_frame(lq_org)
+
+    where <- lq_org$where
+  } else {
+    vars <- op_vars(lq)
+    where <- NULL
+  }
+
+  list(
+    lq = lq,
+    vars = vars,
+    by = by,
+    where = where
+  )
+}
+
+make_join_aliases <- function(con, x_as, y_as, sql_on, call) {
   x_as <- check_join_as1(x_as, arg = "x_as", sql_on, default = "LHS", call)
   y_as <- check_join_as1(y_as, arg = "y_as", sql_on, default = "RHS", call)
 
@@ -501,25 +746,26 @@ make_join_aliases <- function(x_as, y_as, sql_on, call) {
     cli_abort("{.arg y_as} must be different from {.arg x_as}.", call = call)
   }
 
-  list(x = x_as, y = y_as)
+  list(
+    x = if (!is.null(x_as)) as_table_path(x_as, con),
+    y = if (!is.null(y_as)) as_table_path(y_as, con)
+  )
 }
 
 make_table_names <- function(as, lq) {
-  name <- unclass(query_name(lq))
+  name <- remote_name(lq, null_if_local = FALSE)
 
   if (!is.null(as)) {
     tibble(name = as, from = "as")
   } else if (!is.null(name)) {
     tibble(name = name, from = "name")
   } else {
-    tibble(name = NA_character_, from = "")
+    tibble(name = table_path(""), from = "")
   }
 }
 
 check_join_as1 <- function(as, arg, sql_on, default, call) {
-  if (!is_null(as)) {
-    vctrs::vec_assert(as, character(), size = 1, arg = arg, call = call)
-  }
+  check_string(as, allow_null = TRUE, arg = arg, call = call)
 
   if (!is_null(sql_on)) {
     # for backwards compatibility use "LHS"/"RHS" if `sql_on` is used
@@ -530,48 +776,12 @@ check_join_as1 <- function(as, arg, sql_on, default, call) {
   as
 }
 
-check_join_as <- function(x_as, x, y_as, y, sql_on, call) {
-  if (!is_null(x_as)) {
-    vctrs::vec_assert(x_as, character(), size = 1, arg = "x_as", call = call)
-  }
-  if (!is_null(y_as)) {
-    vctrs::vec_assert(y_as, character(), size = 1, arg = "y_as", call = call)
-  }
-
-  if (is_null(sql_on)) {
-    x_name <- unclass(query_name(x))
-    y_name <- unclass(query_name(y))
-    if (is_null(x_as) && is_null(y_as) && identical(x_name, y_name)) {
-      # minor hack to deal with `*_name` = NULL
-      x_as <- paste0(c(x_name, "LHS"), collapse = "_")
-      y_as <- paste0(c(y_name, "RHS"), collapse = "_")
-      # we can safely omit the check that x_as and y_as are identical
-      return(list(x_as = ident(x_as), y_as = ident(y_as)))
-    }
-
-    x_as <- x_as %||% x_name %||% "LHS"
-    y_as <- y_as %||% y_name %||% "RHS"
-  } else {
-    # for backwards compatibility use "LHS" and "RHS" if `sql_on` is used
-    # without a table alias
-    x_as <- x_as %||% "LHS"
-    y_as <- y_as %||% "RHS"
-  }
-
-  if (identical(x_as, y_as)) {
-    cli_abort("{.arg y_as} must be different from {.arg x_as}.", call = call)
-  }
-
-  list(x_as = ident(x_as), y_as = ident(y_as))
-}
-
 join_vars <- function(x_names, y_names, type, by, suffix = c(".x", ".y"), call = caller_env()) {
   y_names_org <- y_names
   # Remove join keys from y
   y_names <- setdiff(y_names, by$y)
 
   # Add suffix where needed
-  suffix <- check_suffix(suffix, call)
   x_new <- add_suffixes(x_names, y_names, suffix$x)
   y_new <- add_suffixes(y_names, x_names, suffix$y)
 
@@ -599,54 +809,75 @@ join_vars <- function(x_names, y_names, type, by, suffix = c(".x", ".y"), call =
   )
 }
 
+# names are bare table names without qualifiers
+# only called from generate_join_table_names
 join_two_table_alias <- function(names, from) {
-  stopifnot(is.character(names))
-  stopifnot(is.character(from))
-  stopifnot(length(names) == 2L)
+  check_character(names)
+  check_character(from)
+  vctrs::vec_assert(names, size = 2L)
 
-  out <- dplyr::coalesce(names, c("LHS", "RHS"))
+  out <- names
+  out[from == ""] <- c("LHS", "RHS")[from == ""]
 
   if (!identical(out[1], out[2])) {
     return(out)
   }
   # -> must rename
 
-  if (from[1] != "as" && from[2] != "as") {
-    # self join of named table
-    if (!is.na(names[1]) && !is.na(names[2]) && identical(names[1], names[2])) {
-      out <- c(
-        paste0(names[1], "_LHS"),
-        paste0(names[2], "_RHS")
-      )
-      return(out)
-    }
+  tables_have_same_name <- from[1] == "name" && from[2] == "name" && identical(names[1], names[2])
+  if (tables_have_same_name) {
+    return(paste0(names, c("_LHS", "_RHS")))
   }
 
   out_repaired <- vctrs::vec_as_names(out, repair = "unique", quiet = TRUE)
-  may_repair <- c(from[1] != "as", from[2] != "as")
-  out[may_repair] <- out_repaired[may_repair]
+  may_repair_name <- c(from[1] != "as", from[2] != "as")
+  out[may_repair_name] <- out_repaired[may_repair_name]
 
   out
 }
 
-check_suffix <- function(x, call) {
-  vctrs::vec_assert(x, character(), size = 2, arg = "suffix", call = call)
-  list(x = x[1], y = x[2])
+check_join_by_supported <- function(by, call = caller_env()) {
+  if (any(by$filter != "none")) {
+    cli_abort("Rolling joins aren't supported on database backends.", call = call)
+  }
 }
 
-add_suffixes <- function(x, y, suffix) {
-  if (identical(suffix, "")) {
-    return(x)
+check_join_multiple <- function(multiple, by, call = caller_env()) {
+  if (is.null(multiple) || identical(multiple, "all")) {
+    return()
   }
 
-  out <- character(length(x))
-  for (i in seq_along(x)) {
-    nm <- x[[i]]
-    while (nm %in% y || nm %in% out) {
-      nm <- paste0(nm, suffix)
-    }
+  msg <- "Argument {.arg multiple} isn't supported on database backends."
 
-    out[[i]] <- nm
+  if (all(by$condition == "==")) {
+    code <- glue::glue("
+      db_create_index(
+        con = remote_con(y),
+        table = remote_name(y),
+        columns = {deparse(by$y)},
+        unique = TRUE
+      )"
+    )
+
+    msg <- c(
+      msg,
+      i = c(
+        "For equi joins you can instead add a unique index for the join columns in {.arg y}.",
+        "{.code {code}}"
+      )
+    )
   }
-  out
+
+  cli_abort(msg, call = call)
+}
+
+check_join_unmatched <- function(unmatched, by, call = caller_env()) {
+  if (is.null(unmatched) || identical(unmatched, "drop")) {
+    return()
+  }
+
+  cli_abort(c(
+    "Argument {.arg unmatched} isn't supported on database backends.",
+    i = "For equi joins you can instead add a foreign key from {.arg x} to {.arg y} for the join columns."
+  ), call = call)
 }
