@@ -38,7 +38,7 @@ test_that("support colwise variants", {
 
 test_that("base source of tbl_lazy is always 'df'", {
   out <- lazy_frame(x = 1, y = 5) %>% sql_build()
-  expect_equal(out, base_query(ident("df")))
+  expect_equal(out, base_query(table_path("`df`")))
 })
 
 test_that("names() inform that they aren't meant to be used", {
